/*
 * Copyright ©Rastislav Mirek. All rights reserved.
 * No waranty for this software or derived works.
 */

package cz.muni.fi.pb162.graphicalobjectexample.graphicalobjects;

import java.awt.Color;

/**
 * This class represents AbstractPolygon.
 *
 * Created 29.10.2009 at 0:56:35.
 *
 * @author Rastislav Mirek <rmirek at mail.muni.cz>
 * @version %I%, %G%
 * @since 1.0
 */
public abstract class AbstractPolygon extends AbstractGraphicalObject
        implements TwoDimensionalGraphicalObject{

    public static final double PRECISION_LOSS_RESERVE = 0.5;

    public AbstractPolygon(Color color){
        super(color);
    }

    protected static double getArea(Coordinate a, Coordinate b, Coordinate c){
        //uses(implements) Heron's formula
        double ab = a.distanceTo(b);
        double bc = b.distanceTo(c);
        double ca = c.distanceTo(a);
        double semiPerimeter = (ab + bc + ca) / 2;
        return Math.sqrt(semiPerimeter * (semiPerimeter - ab) * (semiPerimeter - bc) * (semiPerimeter - ca));
    }

    protected static boolean containsPoint(Coordinate a, Coordinate b, Coordinate c,
            Coordinate inQuestion){
        double abcArea = getArea(a, b, c);
        double abxArea = getArea(a, b, inQuestion);
        double bcxArea = getArea(b, c, inQuestion);
        double caxArea = getArea(c, a, inQuestion);

        //it may never happen that abcArea is larger then the right side of operator >=
        //because of that condition == schould be satisfactiory
        //hovewer this condition is consider more "safe" in programing because prgramers also makes mystakes or may not take something into account
        //and really in this case any other condition would by incorrect because of loss of precision when rounding double values
        //that is also reason for having - PRECISION_LOSS_RESERVE on the right side
        return abcArea >= abxArea + bcxArea + caxArea - PRECISION_LOSS_RESERVE;
    }

    public abstract int vertexCount();

    @Override
    public String toString() {
        return super.toString() + " It represents polygon with " + vertexCount() + " vertexes.";
    }
}
