/*
 * Copyright ©Rastislav Mirek. All rights reserved.
 * No waranty for this software or derived works.
 */

package cz.muni.fi.pb162.graphicalobjectexample.graphicalobjects;

import java.awt.Color;

/**
 * This class represents Circle.
 *
 * Created 28.10.2009 at 14:53:32.
 *
 * @author Rastislav Mirek <rmirek at mail.muni.cz>
 * @version %I%, %G%
 * @since 1.0
 */
public class Circle extends AbstractGraphicalObject implements TwoDimensionalGraphicalObject{

    //constant π is declared here only for example how to use special case of static atributes
    //and how to declare them. π can be normally obtaned as constant defined in Math class
    //in much better accuracy. Note that constants can be declared public and are declared
    //in upper case(capital letters) using _ as word divider
    public static final double PI = 3.14;

    private Coordinate center; // is also position
    private int diameter; //in pixels

    public Circle(Coordinate center, int diameter, Color color) {
        super(color);
        if(center == null){
            throw new NullPointerException("center");
        }
        if(diameter <= 0){
            throw new IllegalArgumentException("Diameter of circle cannot be neither less nor " +
                    "equal to zero.");
        }
        this.center = center;
        this.diameter = diameter;
    }

    public Circle(Coordinate center, Coordinate onPerimeter, Color color){
        this(center, (int)Math.round(center.distanceTo(onPerimeter)), color);
    }


    @Override
    public boolean contains(Coordinate point) {
        if(point == null){
            throw new NullPointerException("point");
        }
        return center.distanceTo(point) <= diameter;
    }

    @Override
    public Coordinate moveRelatively(int relX, int relY) {
        center = new Coordinate(center.getX() + relX, center.getY() + relY);
        return getPosition();
    }

    @Override
    public Coordinate getPosition() {
        return center;
    }

    public double getArea() {
        return PI * diameter * diameter;
    }

    @Override
    public String toString() {
        return super.toString() + " It represents a circle with center in " + center +
                " and diameter " + diameter + ".";
    }



}
