/*
 * Copyright ©Rastislav Mirek. All rights reserved.
 * No waranty for this software or derived works.
 */

package cz.muni.fi.pb162.graphicalobjectexample.graphicalobjects;

import java.awt.Color;

/**
 * This class represents Line.
 *
 * Created 28.10.2009 at 14:13:18.
 *
 * @author Rastislav Mirek <rmirek at mail.muni.cz>
 * @version %I%, %G%
 * @since 1.0
 */
public class Line extends AbstractGraphicalObject{

    private Coordinate a; //is also position
    private Coordinate b;

    public Line(Coordinate a, Coordinate b, Color color) {
        super(color);
        if(a == null){
            throw new NullPointerException("a");
        }
        if(b == null){
            throw new NullPointerException("b");
        }
        if(a.equals(b)){
            throw new IllegalArgumentException("Coordinates of a and b parameters " +
                    "cannot be the same. Both coordinates were: " + a);
        }
        this.a = a;
        this.b = b;
    }

    @Override
    public boolean contains(Coordinate point) {
        double q = (a.getX() - b.getX()) / (a.getY() - b.getY());
        //alternatively(for functional programers) just return here:
//        return point.getX() - a.getX() == Math.round((point.getY() - a.getY()) * q);
        Coordinate canonicP = new Coordinate(point.getX() - a.getX(), point.getY() - a.getY());
        return canonicP.getX() == Math.round(point.getY() * q);
    }

    @Override
    public Coordinate moveRelatively(int relX, int relY) {
        a = new Coordinate(a.getX() + relX, a.getY() + relY);
        b = new Coordinate(b.getX() + relX, b.getY() + relY);
        
        return getPosition();
    }

    @Override
    public Coordinate getPosition() {
        return a;
    }

    @Override
    public String toString() {
        return super.toString() + " It represents a line defined by this points: "
                + a + ", " + b + ".";
    }


}
