/*
 * Copyright ©Rastislav Mirek. All rights reserved.
 * No waranty for this software or derived works.
 */

package cz.muni.fi.pb162.graphicalobjectexample.graphicalobjects;

import java.awt.Color;

/**
 * This class represents Tetragon.
 * Even if it might be suprising there is no need check whether the given coordinates
 * create complex tetragon(in constructor). The correct behavior(no support for complex tetragon) is
 * insured by implementation of methods contains and getArea.
 *
 * Created 29.10.2009 at 0:00:52.
 *
 * @author Rastislav Mirek <rmirek at mail.muni.cz>
 * @version %I%, %G%
 * @since 1.0
 */
public class Tetragon extends AbstractPolygon{

    private Coordinate a;
    private Coordinate b;
    private Coordinate c;
    private Coordinate d;

    public static final int NUMBER_OF_VERTEXES = 4;

    public Tetragon(Coordinate a, Coordinate b, Coordinate c, Coordinate d, Color color) {
        super(color);
        if(a == null){
            throw new NullPointerException("a");
        }
        if(b == null){
            throw new NullPointerException("b");
        }
        if(c == null){
            throw new NullPointerException("c");
        }
        if(d == null){
            throw new NullPointerException("d");
        }
        if(a.equals(b) || a.equals(c) || a.equals(d) || b.equals(c) || b.equals(d) ||
                c.equals(d)){
            throw new IllegalArgumentException("Some vertexes of Tetragon are the same." +
                    "Vertextes were: " + a + ", " + b + ", " + c + ", " + d + ".");
        }
        this.a = a;
        this.b = b;
        this.c = c;
        this.d = d;
    }



    @Override
    public boolean contains(Coordinate point) {
        //virtualy split up tetragon to 2 triangles ABC and CDA
        //check if CDA is part of ABC (the case of concave tetragon with concave vertex D)
        //or ABC is part of CDA (the case of concave tetragon with concave vertex B)
        if(containsPoint(a, b, c, d)){
            //ABCD is concave with concave vertex D
            //ABCD contains point if and only if triangle ABC contains point but triangle CDA does not
            return containsPoint(a, b, c, point) && !containsPoint(c, d, a, point);
        }
        else if(containsPoint(c, d, a, b)){
            //ABCD is concave with concave vertex B
            //ABCD contains point if and only if triangle CDA contains point but triangle ABC does not
            return containsPoint(c, d, a, point) && ! containsPoint(a, b, c, point);
        }
        else{
            //ABCD is either convex or concave with concave vertex A or D
            //ABCD contains point if point is contained inone of triangles ABC or CDA
            return containsPoint(a, b, c, point) || containsPoint(c, d, a, point);
        }
    }

    @Override
    public Coordinate moveRelatively(int relX, int relY) {
        a = new Coordinate(a.getX() + relX, a.getY() + relY);
        b = new Coordinate(b.getX() + relX, b.getY() + relY);
        c = new Coordinate(c.getX() + relX, c.getY() + relY);
        d = new Coordinate(d.getX() + relX, d.getY() + relY);

        return getPosition();
    }

    @Override
    public Coordinate getPosition() {
        return a;
    }

    public double getArea() {
        //notes on algorithm analogical to those in method contains
        if(containsPoint(a, b, c, d)){
            //D is concave vertex
            return getArea(a, b, c) - getArea(c, d, a);
        }
        else if(containsPoint(c, d, a, b)){
            //B is concave vertex
            return getArea(c, d, a) - getArea(a, b, c);
        }
        else{
            return getArea(a, b, c) + getArea(c, d, a);
        }
    }

    public boolean isConcave(){
        return containsPoint(b, c, d, a) || containsPoint(a, c, d, b) ||
                containsPoint(a, b, d, c) || containsPoint(a, b, c, d);
    }

    @Override
    public int vertexCount() {
        return NUMBER_OF_VERTEXES;
    }

    @Override
    public String toString() {
        return super.toString() + " Vertexes of this polygon are: " + a
                + ", " + b + ", " + c + ", " + d + ".";
    }

    protected Coordinate getA() {
        return a;
    }

}
