/*
 * Copyright ©Rastislav Mirek. All rights reserved.
 * No waranty for this software or derived works.
 */

package cz.muni.fi.pb162.graphicalobjectexample.graphicalobjects;

import java.awt.Color;

/**
 * This class represents Triangle.
 *
 * Created 29.10.2009 at 0:15:47.
 *
 * @author Rastislav Mirek <rmirek at mail.muni.cz>
 * @version %I%, %G%
 * @since 1.0
 */
public class Triangle extends AbstractPolygon{
    
    private Coordinate a;
    private Coordinate b;
    private Coordinate c;

    public static final int NUMBER_OF_VERTEXES = 3;

    public Triangle(Coordinate a, Coordinate b, Coordinate c, Color color) {
        super(color);
        if(a == null){
            throw new NullPointerException("a");
        }
        if(b == null){
            throw new NullPointerException("b");
        }
        if(c == null){
            throw new NullPointerException("c");
        }
        if(a.equals(b) || b.equals(c) || c.equals(a)){
            throw new IllegalArgumentException("Two points of the triangle with vertexes: "
                    + a + ", " + b + ", " + c + " are the same.");
        }
        this.a = a;
        this.b = b;
        this.c = c;
    }

    @Override
    public boolean contains(Coordinate point) {
        return containsPoint(a, b, c, point);
    }

    @Override
    public Coordinate getPosition() {
        return a;
    }

    @Override
    public Coordinate moveRelatively(int relX, int relY) {
        a = new Coordinate(a.getX() + relX, a.getY() + relY);
        b = new Coordinate(b.getX() + relX, b.getY() + relY);
        c = new Coordinate(c.getX() + relX, c.getY() + relY);

        return getPosition();
    }

    public double getArea(){
        return getArea(a, b, c);
    }



    public double getPerimeter(){
        return a.distanceTo(b) + b.distanceTo(c) + c.distanceTo(a);
    }

    @Override
    public int vertexCount() {
        return NUMBER_OF_VERTEXES;
    }

    @Override
    public String toString() {
        return super.toString() + " The vertexes are " + a + ", " + b + ", " + c + ".";
    }

}
